@@ -2,18 +2,18 @@
 ===============================================================================
 
   FILE:  lasreadpoint.cpp
-  
+
   CONTENTS:
-  
+
     see corresponding header file
-  
+
   PROGRAMMERS:
 
     martin.isenburg@rapidlasso.com  -  http://rapidlasso.com
 
   COPYRIGHT:
 
-    (c) 2007-2014, martin isenburg, rapidlasso - fast tools to catch reality
+    (c) 2007-2017, martin isenburg, rapidlasso - fast tools to catch reality
 
     This is free software; you can redistribute and/or modify it under the
     terms of the GNU Lesser General Licence as published by the Free Software
@@ -21,11 +21,11 @@
 
     This software is distributed WITHOUT ANY WARRANTY and without even the
     implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-  
+
   CHANGE HISTORY:
-  
+
     see corresponding header file
-  
+
 ===============================================================================
 */
 
@@ -35,12 +35,13 @@
 #include "lasreaditemraw.hpp"
 #include "lasreaditemcompressed_v1.hpp"
 #include "lasreaditemcompressed_v2.hpp"
+#include "lasreaditemcompressed_v3.hpp"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-LASreadPoint::LASreadPoint()
+LASreadPoint::LASreadPoint(U32 decompress_selective)
 {
   point_size = 0;
   instream = 0;
@@ -49,6 +50,7 @@ LASreadPoint::LASreadPoint()
   readers_raw = 0;
   readers_compressed = 0;
   dec = 0;
+  layered_las14_compression = FALSE;
   // used for chunking
   chunk_size = U32_MAX;
   chunk_count = 0;
@@ -57,6 +59,8 @@ LASreadPoint::LASreadPoint()
   tabled_chunks = 0;
   chunk_totals = 0;
   chunk_starts = 0;
+  // used for selective decompression (new LAS 1.4 point types only)
+  this->decompress_selective = decompress_selective;
   // used for seeking
   point_start = 0;
   seek_point = 0;
@@ -72,18 +76,24 @@ BOOL LASreadPoint::setup(U32 num_items, const LASitem* items, const LASzip* lasz
   // is laszip exists then we must use its items
   if (laszip)
   {
+    if (num_items == 0) return FALSE;
+    if (items == 0) return FALSE;
     if (num_items != laszip->num_items) return FALSE;
     if (items != laszip->items) return FALSE;
   }
 
-  // create entropy decoder (if requested)
+  // delete old entropy decoder
   if (dec)
   {
     delete dec;
     dec = 0;
+    layered_las14_compression = FALSE;
   }
+
+  // is the content compressed?
   if (laszip && laszip->compressor)
   {
+    // create new entropy decoder (if requested)
     switch (laszip->coder)
     {
     case LASZIP_CODER_ARITHMETIC:
@@ -93,8 +103,10 @@ BOOL LASreadPoint::setup(U32 num_items, const LASitem* items, const LASzip* lasz
       // entropy decoder not supported
       return FALSE;
     }
+    // maybe layered compression for LAS 1.4
+    layered_las14_compression = (laszip->compressor == LASZIP_COMPRESSOR_LAYERED_CHUNKED);
   }
- 
+
   // initizalize the readers
   readers = 0;
   num_readers = num_items;
@@ -121,25 +133,21 @@ BOOL LASreadPoint::setup(U32 num_items, const LASitem* items, const LASzip* lasz
         readers_raw[i] = new LASreadItemRaw_GPSTIME11_BE();
         break;
     case LASitem::RGB12:
+    case LASitem::RGB14:
       if (IS_LITTLE_ENDIAN())
         readers_raw[i] = new LASreadItemRaw_RGB12_LE();
       else
         readers_raw[i] = new LASreadItemRaw_RGB12_BE();
       break;
-    case LASitem::WAVEPACKET13:
-      if (IS_LITTLE_ENDIAN())
-        readers_raw[i] = new LASreadItemRaw_WAVEPACKET13_LE();
-      else
-        readers_raw[i] = new LASreadItemRaw_WAVEPACKET13_BE();
-      break;
     case LASitem::BYTE:
+    case LASitem::BYTE14:
       readers_raw[i] = new LASreadItemRaw_BYTE(items[i].size);
       break;
     case LASitem::POINT14:
       if (IS_LITTLE_ENDIAN())
         readers_raw[i] = new LASreadItemRaw_POINT14_LE();
       else
-        return FALSE;
+        readers_raw[i] = new LASreadItemRaw_POINT14_BE();
       break;
     case LASitem::RGBNIR14:
       if (IS_LITTLE_ENDIAN())
@@ -147,6 +155,13 @@ BOOL LASreadPoint::setup(U32 num_items, const LASitem* items, const LASzip* lasz
       else
         readers_raw[i] = new LASreadItemRaw_RGBNIR14_BE();
       break;
+    case LASitem::WAVEPACKET13:
+    case LASitem::WAVEPACKET14:
+      if (IS_LITTLE_ENDIAN())
+        readers_raw[i] = new LASreadItemRaw_WAVEPACKET13_LE();
+      else
+        readers_raw[i] = new LASreadItemRaw_WAVEPACKET13_BE();
+      break;
     default:
       return FALSE;
     }
@@ -194,12 +209,6 @@ BOOL LASreadPoint::setup(U32 num_items, const LASitem* items, const LASzip* lasz
         else
           return FALSE;
         break;
-      case LASitem::WAVEPACKET13:
-        if (items[i].version == 1)
-          readers_compressed[i] = new LASreadItemCompressed_WAVEPACKET13_v1(dec);
-        else
-          return FALSE;
-        break;
       case LASitem::BYTE:
         if (items[i].version == 1)
           readers_compressed[i] = new LASreadItemCompressed_BYTE_v1(dec, items[i].size);
@@ -208,12 +217,48 @@ BOOL LASreadPoint::setup(U32 num_items, const LASitem* items, const LASzip* lasz
         else
           return FALSE;
         break;
+      case LASitem::POINT14:
+        if ((items[i].version == 3) || (items[i].version == 2)) // version == 2 from lasproto
+          readers_compressed[i] = new LASreadItemCompressed_POINT14_v3(dec, decompress_selective);
+        else
+          return FALSE;
+        break;
+      case LASitem::RGB14:
+        if ((items[i].version == 3) || (items[i].version == 2)) // version == 2 from lasproto
+          readers_compressed[i] = new LASreadItemCompressed_RGB14_v3(dec, decompress_selective);
+        else
+          return FALSE;
+        break;
+      case LASitem::RGBNIR14:
+        if ((items[i].version == 3) || (items[i].version == 2)) // version == 2 from lasproto
+          readers_compressed[i] = new LASreadItemCompressed_RGBNIR14_v3(dec, decompress_selective);
+        else
+          return FALSE;
+        break;
+      case LASitem::BYTE14:
+        if ((items[i].version == 3) || (items[i].version == 2)) // version == 2 from lasproto
+          readers_compressed[i] = new LASreadItemCompressed_BYTE14_v3(dec, items[i].size, decompress_selective);
+        else
+          return FALSE;
+        break;
+      case LASitem::WAVEPACKET13:
+        if (items[i].version == 1)
+          readers_compressed[i] = new LASreadItemCompressed_WAVEPACKET13_v1(dec);
+        else
+          return FALSE;
+        break;
+      case LASitem::WAVEPACKET14:
+        if (items[i].version == 3)
+          readers_compressed[i] = new LASreadItemCompressed_WAVEPACKET14_v3(dec, decompress_selective);
+        else
+          return FALSE;
+        break;
       default:
         return FALSE;
       }
       if (i) seek_point[i] = seek_point[i-1]+items[i-1].size;
     }
-    if (laszip->compressor == LASZIP_COMPRESSOR_POINTWISE_CHUNKED)
+    if (laszip->compressor != LASZIP_COMPRESSOR_POINTWISE)
     {
       if (laszip->chunk_size) chunk_size = laszip->chunk_size;
       number_chunks = U32_MAX;
@@ -382,10 +427,34 @@ BOOL LASreadPoint::read(U8* const * point)
         for (i = 0; i < num_readers; i++)
         {
           readers_raw[i]->read(point[i]);
-          ((LASreadItemCompressed*)(readers_compressed[i]))->init(point[i]);
+        }
+        if (layered_las14_compression)
+        {
+          // for layered compression 'dec' only hands over the stream
+          dec->init(instream, FALSE);
+          // read how many points are in the chunk
+          U32 count;
+          instream->get32bitsLE((U8*)&count);
+          // read the sizes of all layers
+          for (i = 0; i < num_readers; i++)
+          {
+            ((LASreadItemCompressed*)(readers_compressed[i]))->chunk_sizes();
+          }
+          for (i = 0; i < num_readers; i++)
+          {
+            ((LASreadItemCompressed*)(readers_compressed[i]))->init(point[i]);
+          }
+          if (DEBUG_OUTPUT_NUM_BYTES_DETAILS) REprintf( "\n");
+        }
+        else
+        {
+          for (i = 0; i < num_readers; i++)
+          {
+            ((LASreadItemCompressed*)(readers_compressed[i]))->init(point[i]);
+          }
+          dec->init(instream);
         }
         readers = readers_compressed;
-        dec->init(instream);
       }
     }
     else
@@ -396,7 +465,7 @@ BOOL LASreadPoint::read(U8* const * point)
       }
     }
   }
-  catch (I32 exception) 
+  catch (I32 exception)
   {
     // create error string
     if (last_error == 0) last_error = new CHAR[128];
@@ -524,7 +593,7 @@ BOOL LASreadPoint::read_chunk_table()
       return FALSE;
     }
     // then we cannot seek to the chunk table but won't need it anyways
-    number_chunks = U32_MAX-1;
+    number_chunks = 0;
     tabled_chunks = 0;
     return TRUE;
   }
